import { useAtomValue } from 'jotai'
import React, { useEffect, useRef, useState } from 'react'
import { View, useWindowDimensions } from 'react-native'

import {
  DEFAULT_DISTANCE_BELOW_FOLD_THRESHOLD,
  DEFAULT_PLACEHOLDER_HEIGHT,
  ScrollBasedCallbackInvocationProps,
} from '@holi/core/components/ScrollBasedCallbackInvocation.shared'
import { mobileScrollYAtom } from '@holi/core/helpers'
import { useOnRefresh } from '@holi/core/refreshing/hooks/useOnRefresh'

const ScrollBasedCallbackInvocation = ({
  belowFoldThreshold = DEFAULT_DISTANCE_BELOW_FOLD_THRESHOLD,
  placeholder,
  callback,
}: ScrollBasedCallbackInvocationProps) => {
  const [lastPosition, setLastPosition] = useState(0)
  const placeholderRef = useRef<View>(null)
  const { height: windowHeight } = useWindowDimensions()

  // scroll-based check for visibility
  const scrollY = useAtomValue(mobileScrollYAtom)

  // initial & re-render based check for visibility
  useEffect(() => {
    // react native only returns zeros when we ask too early -_-
    setTimeout(() => {
      placeholderRef.current?.measure((x, y, width, height, pageX, pageY) => {
        if (lastPosition >= scrollY + belowFoldThreshold) {
          // short circuit when previous call already triggered
          return
        }

        const distanceBelowFold = windowHeight - pageY
        if (distanceBelowFold > belowFoldThreshold) {
          setLastPosition(scrollY)
          callback()
        }
      })
    }, 0)
  }, [windowHeight, belowFoldThreshold, scrollY, callback, lastPosition])

  useOnRefresh(() => {
    // Reset scroll position when refreshing content, so e.g. pagination can start anew
    setLastPosition(0)
  })

  return (
    <View ref={placeholderRef} collapsable={false}>
      {placeholder ?? <View style={{ height: DEFAULT_PLACEHOLDER_HEIGHT }} />}
    </View>
  )
}

export default ScrollBasedCallbackInvocation
